package org.ioc.commons.flowcontrol.eventbus;

import org.ioc.commons.flowcontrol.common.BindRegistration;

/**
 * Binder for events
 * 
 * @author Jesús Lunar Pérez
 * 
 * @param <E>
 *            Event type
 */
public interface EventBusBinder<E extends Enum<? extends IsEvent>> {
	/**
	 * Binds an event to the handler which will process it when it is fired.
	 * 
	 * @param event
	 *            Event to bind
	 * @param handler
	 *            Handler which will process the event
	 * 
	 * @return A bind registration object for managing the new binding
	 *         registered.
	 */
	BindRegistration bindEvent(E event, EventHandler<E> handler);
}
