package org.ioc.commons.flowcontrol.operationmanager;

/**
 * Handler for operations
 * 
 * @author Jesús Lunar Pérez
 * 
 * @param <O>
 *            Operation type
 */
public interface OperationHandler<O extends Enum<? extends IsOperation>> {
	/**
	 * Method which will be called when a bound operation is begun.
	 * 
	 * @param sender
	 *            Entity which is beginning the operation
	 * @param operation
	 *            Operation begun
	 */
	void onOperationBeginning(Object sender, O operation);

	/**
	 * Method which will be called when a bound operation is finished.
	 * 
	 * @param sender
	 *            Entity which communicates the operation has finished.
	 * @param operation
	 *            Operation finished.
	 * @param success
	 *            Indicates if the operation finished successfuly or not.
	 */
	void onOperationFinished(Object sender, O operation, boolean success);
}
