package org.ioc.commons.flowcontrol.operationmanager.impl;

import org.ioc.commons.flowcontrol.operationmanager.IsOperation;
import org.ioc.commons.flowcontrol.operationmanager.OperationHandler;
import org.ioc.commons.ui.HasLoader;

/**
 * It wraps an operation handler from some HasLoaders.
 * 
 * Instanciate from {@link OperationHandlerFactory}
 * 
 * @author Jesús Lunar Pérez
 *
 * @param <O>
 */
public class HasLoaderOperationHandler<O extends Enum<? extends O> & IsOperation> implements OperationHandler<O> {

	private HasLoader[] loaders;

	HasLoaderOperationHandler(HasLoader... loaders) {
		this.loaders = loaders;
	}

	@Override
	public void onOperationBeginning(Object sender, O operation) {
		if (this.loaders != null) {
			for (HasLoader loader : loaders) {
				loader.setLoading(true);
			}
		}
	}

	@Override
	public void onOperationFinished(Object sender, O operation, boolean success) {
		if (this.loaders != null) {
			for (HasLoader loader : loaders) {
				loader.setLoading(false);
			}
		}
	}
}
