package org.ioc.commons.impl.gwt.client.flowcontrol.actioncontroller;

import org.ioc.commons.flowcontrol.actioncontroller.ActionController;
import org.ioc.commons.flowcontrol.actioncontroller.ActionHandler;
import org.ioc.commons.flowcontrol.actioncontroller.IsExternalAction;
import org.ioc.commons.flowcontrol.common.BindRegistration;

import com.google.web.bindery.event.shared.HandlerRegistration;

public class GwtActionControllerImpl<A extends Enum<? extends IsExternalAction>> implements ActionController<A> {

	private static class GwtActionControllerBindRegistration implements BindRegistration {

		private HandlerRegistration invokedActionHandlerRegistration;

		public GwtActionControllerBindRegistration(HandlerRegistration invokedActionHandlerRegistration) {
			this.invokedActionHandlerRegistration = invokedActionHandlerRegistration;
		}

		@Override
		public void unbind() {
			this.invokedActionHandlerRegistration.removeHandler();
		}

	}

	private com.google.web.bindery.event.shared.EventBus eventBus;

	public GwtActionControllerImpl(com.google.web.bindery.event.shared.EventBus eventBus) {
		this.eventBus = eventBus;
	}

	@Override
	public BindRegistration bindAction(final A action, final ActionHandler<A> handler) {
		HandlerRegistration handlerRegistration = this.eventBus.addHandler(GwtActionInvokedEvent.TYPE,
				new GwtActionInvokedEvent.Handler() {

					@Override
					public void onActionInvoked(GwtActionInvokedEvent<? extends Enum<? extends IsExternalAction>> event) {
						if (event.getAction().equals(action)) {
							handler.handleAction(event.getSource(), action, event.getParams());
						}

					}
				});

		GwtActionControllerBindRegistration registration = new GwtActionControllerBindRegistration(handlerRegistration);

		return registration;
	}

	@Override
	public void invokeAction(A action, Object... params) {
		eventBus.fireEvent(new GwtActionInvokedEvent<Enum<? extends IsExternalAction>>(action, params));
	}

}
