package org.ioc.commons.impl.gwt.client.flowcontrol.actioncontroller;

import org.ioc.commons.flowcontrol.actioncontroller.IsExternalAction;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.web.bindery.event.shared.EventBus;
import com.google.web.bindery.event.shared.HandlerRegistration;

class GwtActionInvokedEvent<A extends Enum<? extends IsExternalAction>> extends GwtEvent<GwtActionInvokedEvent.Handler> {

	public static HandlerRegistration register(EventBus eventBus, GwtActionInvokedEvent.Handler handler) {
		return eventBus.addHandler(TYPE, handler);
	}

	public static <A extends Enum<? extends IsExternalAction>> void fire(EventBus eventBus, A action, Object source) {
		if (TYPE != null) {
			GwtActionInvokedEvent<A> event = new GwtActionInvokedEvent<A>(action);
			eventBus.fireEventFromSource(event, source);
		}
	}

	private A action;
	private Object[] params;

	GwtActionInvokedEvent(A action, Object... params) {
		this.action = action;
		this.params = params;
	}

	/**
	 * Implemented by objects that handle {@link GwtActionInvokedEvent}.
	 */
	public interface Handler extends EventHandler {
		void onActionInvoked(GwtActionInvokedEvent<? extends Enum<? extends IsExternalAction>> event);
	}

	/**
	 * The event type.
	 */
	public static final Type<GwtActionInvokedEvent.Handler> TYPE = new Type<GwtActionInvokedEvent.Handler>();

	@Override
	public final Type<GwtActionInvokedEvent.Handler> getAssociatedType() {
		return TYPE;
	}

	@Override
	protected void dispatch(GwtActionInvokedEvent.Handler handler) {
		handler.onActionInvoked(this);
	}

	public A getAction() {
		return action;
	}
	
	public Object[] getParams() {
		return params;
	}
}
