package org.ioc.commons.impl.gwt.client.flowcontrol.eventbus;

import org.ioc.commons.flowcontrol.common.BindRegistration;
import org.ioc.commons.flowcontrol.eventbus.EventBus;
import org.ioc.commons.flowcontrol.eventbus.EventHandler;
import org.ioc.commons.flowcontrol.eventbus.IsEvent;

import com.google.web.bindery.event.shared.HandlerRegistration;

public class GwtEventBusImpl<E extends Enum<? extends IsEvent>> implements EventBus<E> {

	private com.google.web.bindery.event.shared.EventBus gwtEventBus;

	private static class GwtEventBusBindRegistration implements BindRegistration {

		private HandlerRegistration eventBusHandlerRegistration;

		public GwtEventBusBindRegistration(HandlerRegistration eventBusHandlerRegistration) {
			this.eventBusHandlerRegistration = eventBusHandlerRegistration;
		}

		@Override
		public void unbind() {
			this.eventBusHandlerRegistration.removeHandler();
		}

	}

	public GwtEventBusImpl(com.google.web.bindery.event.shared.EventBus eventBus) {
		assert(eventBus!=null) : "El handler manager no puede ser nulo";
		this.gwtEventBus = eventBus;
	}

	@Override
	public BindRegistration bindEvent(final E event, final EventHandler<E> handler) {
		HandlerRegistration handlerRegistration = this.gwtEventBus.addHandler(GwtEventFiredEvent.TYPE,
				new GwtEventFiredEvent.Handler() {

					@Override
					public void onEventFired(GwtEventFiredEvent<? extends Enum<? extends IsEvent>> gwtEvent) {
						if (gwtEvent.getEvent().equals(event)) {
							handler.handleEvent(gwtEvent.getSource(), event, gwtEvent.getParams());
						}
					}
				});

		return new GwtEventBusBindRegistration(handlerRegistration);
	}

	@Override
	public void fireEvent(E event, Object... params) {
		this.gwtEventBus.fireEvent(new GwtEventFiredEvent<E>(event, params));
	}

	@Override
	public Object asEventBus() {
		return gwtEventBus;
	}

}
