package org.ioc.commons.impl.gwt.client.flowcontrol.placecontroller;

import org.ioc.commons.flowcontrol.eventbus.EventBusBinder;
import org.ioc.commons.flowcontrol.placecontroller.HasPlaceData;
import org.ioc.commons.flowcontrol.placecontroller.IsPlace;
import org.ioc.commons.flowcontrol.placecontroller.PlaceController;
import org.ioc.commons.impl.gwt.client.flowcontrol.eventbus.GwtEventBusImpl;

import com.google.gwt.place.shared.Place;
import com.google.gwt.place.shared.PlaceChangeEvent;
import com.google.gwt.place.shared.PlaceHistoryHandler;
import com.google.gwt.place.shared.PlaceHistoryMapper;
import com.google.gwt.user.client.History;
import com.google.web.bindery.event.shared.EventBus;

public class GwtPlaceController extends com.google.gwt.place.shared.PlaceController implements PlaceController {

	public interface PlaceMapper {
		<P extends Enum<? extends IsPlace>> Place getPlace(P place, Object... params);
	}

	private static boolean strictMode = true;

	private PlaceMapper mapper;
	private PlaceHistoryHandler historyHandler;
	private EventBus gwtEventBus;
	private PlaceHistoryMapper gwtHistoryMapper;

	private GwtEventBusImpl<Event> eventBus;

	public GwtPlaceController(PlaceMapper placeMapper, PlaceHistoryMapper gwtHistoryMapper, EventBus eventBus,
			Delegate delegate) {
		super(eventBus, delegate);
		init(placeMapper, gwtHistoryMapper, eventBus);
	}

	public GwtPlaceController(PlaceMapper placeMapper, PlaceHistoryMapper gwtHistoryMapper, EventBus eventBus) {
		super(eventBus);
		init(placeMapper, gwtHistoryMapper, eventBus);
	}

	public void init(PlaceMapper placeMapper, PlaceHistoryMapper gwtHistoryMapper, EventBus eventBus) {
		this.gwtHistoryMapper = gwtHistoryMapper;
		this.historyHandler = new PlaceHistoryHandler(gwtHistoryMapper);
		this.gwtEventBus = eventBus;
		this.mapper = placeMapper;
		this.eventBus = new GwtEventBusImpl<Event>(this.gwtEventBus);

		this.gwtEventBus.addHandler(PlaceChangeEvent.TYPE, new PlaceChangeEvent.Handler() {

			@Override
			public void onPlaceChange(PlaceChangeEvent event) {
				if (event.getNewPlace() instanceof HasPlaceData) {
					HasPlaceData hpd = (HasPlaceData) event.getNewPlace();
					GwtPlaceController.this.eventBus.fireEvent(Event.PlaceChanged, hpd.getPlaceId());
				}
			}
		});
	}

	@Override
	public <P extends Enum<? extends IsPlace>> void initPlaceController(P firstPlaceId, Object... params) {
		Place defaultPlace = this.mapper.getPlace(firstPlaceId, params);
		assert (defaultPlace != null) : "The default cannot be null";
		historyHandler.register((com.google.gwt.place.shared.PlaceController) this, this.gwtEventBus, defaultPlace);
	}

	@Override
	public <P extends Enum<? extends IsPlace>> void goTo(P place, Object... params) {
		Place gwtPlace = this.mapper.getPlace(place, params);
		this.goTo(gwtPlace);
	}

	@Override
	public void goTo(HasPlaceData place) {
		assert (place instanceof Place) : "El par�metro debe ser una instancia de " + Place.class;

		Place gwtPlace = (Place) place;
		this.goTo(gwtPlace);
	}

	@Override
	public HasPlaceData getCurrentPlaceData() {
		Place where = this.getWhere();
		assert (!strictMode || (where == Place.NOWHERE) || (where instanceof HasPlaceData)) : "No est� permitido introducir un Place que no herede de HasPlaceData en modo estricto. Desactive el modo estricto con GwtPlaceController.setStrictMode(false) para permitirlo. Where="
				+ where;

		return (where instanceof HasPlaceData) ? (HasPlaceData) where : null;
	}

	@Override
	public void handleCurrentPlace() {
		historyHandler.handleCurrentHistory();
	}

	@Override
	public void appendPlaceInHistory(HasPlaceData place) {
		History.newItem(this.gwtHistoryMapper.getToken((Place) place), false);
	}

	public static boolean isStrictMode() {
		return strictMode;
	}

	public static void setStrictMode(boolean strictMode) {
		GwtPlaceController.strictMode = strictMode;
	}

	@Override
	public <P extends Enum<? extends IsPlace>> HasPlaceData getPlaceData(P placeId, Object... params) {
		Place gwtPlace = this.mapper.getPlace(placeId, params);
		return (gwtPlace instanceof HasPlaceData) ? (HasPlaceData) gwtPlace : null;
	}

	@Override
	public EventBusBinder<Event> eventbus() {
		return this.eventBus;
	}

	@Override
	public <P extends Enum<? extends IsPlace>> void initPlaceController(boolean handleCurrentPlaceNow, P firstPlaceId,
			Object... params) {
		this.initPlaceController(firstPlaceId, params);
		if (handleCurrentPlaceNow) {
			this.handleCurrentPlace();
		}
	}
}
