package org.ioc.commons.impl.gwt.client.flowcontrol.taskcontroller;

import org.ioc.commons.flowcontrol.taskcontroller.RepeatingTask;
import org.ioc.commons.flowcontrol.taskcontroller.Task;
import org.ioc.commons.flowcontrol.taskcontroller.TaskController;
import org.ioc.commons.flowcontrol.taskcontroller.TaskHandler;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.RunAsyncCallback;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.user.client.Timer;

public class GwtTaskController implements TaskController {

	private static class NonCancellableTaskWrapper implements TaskHandler {

		@Override
		public boolean cancel() {
			return false;
		}

	}

	private static class TimerTaskWrapper implements TaskHandler {
		private Task task;
		private Timer timer = new Timer() {

			@Override
			public void run() {
				TimerTaskWrapper.this.task.run();
			}
		};

		public TimerTaskWrapper(Task task) {
			this.task = task;
		}

		@Override
		public boolean cancel() {
			this.timer.cancel();
			return true;
		}
	}

	private static final TaskHandler NON_CANCELLABLE_TASK_WRAPPER = new NonCancellableTaskWrapper();

	//TODO: Hacer singleton
	public GwtTaskController() {
		// Nada
	}

	@Override
	public TaskHandler scheduleDelayed(int delayMilis, Task task) {
		TimerTaskWrapper timerTaskWrapper = new TimerTaskWrapper(task);
		timerTaskWrapper.timer.schedule(delayMilis);

		return timerTaskWrapper;
	}

	@Override
	public TaskHandler runAsync(final Task task) {
		GWT.runAsync(new RunAsyncCallback() {

			@Override
			public void onSuccess() {
				task.run();
			}

			@Override
			public void onFailure(Throwable reason) {
				GWT.getUncaughtExceptionHandler().onUncaughtException(reason);
			}
		});

		return NON_CANCELLABLE_TASK_WRAPPER;
	}

	@Override
	public TaskHandler scheduleFinally(final Task task) {
		Scheduler.get().scheduleFinally(new Scheduler.ScheduledCommand() {

			@Override
			public void execute() {
				task.run();
			}
		});
		
		return NON_CANCELLABLE_TASK_WRAPPER;
	}

	@Override
	public TaskHandler scheduleFinally(final RepeatingTask task) {
		Scheduler.get().scheduleFinally(new Scheduler.RepeatingCommand() {

			@Override
			public boolean execute() {
				return task.run();
			}
		});
		
		return NON_CANCELLABLE_TASK_WRAPPER;
	}

	@Override
	public TaskHandler scheduleIdle(final Task task) {
		Scheduler.get().scheduleDeferred(new Scheduler.ScheduledCommand() {

			@Override
			public void execute() {
				task.run();
			}
		});
		
		return NON_CANCELLABLE_TASK_WRAPPER;
	}

	@Override
	public TaskHandler scheduleIdle(final RepeatingTask task) {
		Scheduler.get().scheduleIncremental(new Scheduler.RepeatingCommand() {

			@Override
			public boolean execute() {
				return task.run();
			}
		});
		
		return NON_CANCELLABLE_TASK_WRAPPER;
	}

}
