package org.ioc.commons.impl.gwt.client.ui;

import org.ioc.commons.ui.IsWidget;
import org.ioc.commons.ui.OneChildContainer;

import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.Widget;

public class OneChildContainerWrapper<T extends com.google.gwt.user.client.ui.HasOneWidget & com.google.gwt.user.client.ui.IsWidget>
		implements OneChildContainer {

	private static final class RootPanelOneChildContainerWrapper implements com.google.gwt.user.client.ui.HasOneWidget,
			com.google.gwt.user.client.ui.IsWidget {
		private final RootPanel rootPanel;
		private Widget widget;

		private RootPanelOneChildContainerWrapper(RootPanel rootPanel) {
			this.rootPanel = rootPanel;
		}

		@Override
		public void setWidget(com.google.gwt.user.client.ui.IsWidget w) {
			rootPanel.clear();
			rootPanel.add(w);
			this.widget = w.asWidget();
		}

		@Override
		public void setWidget(Widget w) {
			rootPanel.clear();
			rootPanel.add(w);
			this.widget = w;
		}

		@Override
		public Widget getWidget() {
			return widget;
		}

		@Override
		public Widget asWidget() {
			return rootPanel;
		}
	}

	private static class IsWidgetImpl implements IsWidget {

		private Widget widget;

		public IsWidgetImpl(Widget widget) {
			this.widget = widget;
		}

		@Override
		public Widget asWidget() {
			return widget;
		}

	}

	private T wrapped;

	private OneChildContainerWrapper(T wrapped) {
		this.wrapped = wrapped;
	}

	@Override
	public void setWidget(IsWidget widget) {
		this.wrapped.setWidget((Widget) widget.asWidget());
	}

	@Override
	public IsWidget getWidget() {
		return new IsWidgetImpl(this.wrapped.getWidget());
	}

	public static <T extends com.google.gwt.user.client.ui.HasOneWidget & com.google.gwt.user.client.ui.IsWidget> OneChildContainer from(
			T gwtOneChildContainer) {
		return new OneChildContainerWrapper<T>(gwtOneChildContainer);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static OneChildContainer from(final RootPanel rootPanel) {
		return new OneChildContainerWrapper(new RootPanelOneChildContainerWrapper(rootPanel));
	}

	@Override
	public Object asWidget() {
		return this.wrapped;
	}

}
