package java.util;

import java.util.HashSet;
import java.util.Set;

public class Observable {

	private final Set<Observer> observers;
	private boolean changed;

	public Observable() {
		this.observers = new HashSet<Observer>();
		this.changed = false;
	}

	public void addObserver(Observer o) {
		if (o == null) {
			throw new NullPointerException("The observer cannot be null.");
		}
		synchronized (this) {
			this.observers.add(o);
		}
	}

	public int countObservers() {
		synchronized (this) {
			return this.observers.size();
		}
	}

	public void deleteObserver(Observer o) {
		synchronized (this) {
			this.observers.remove(o);
		}
	}

	public void deleteObservers() {
		synchronized (this) {
			this.observers.clear();
		}
	}

	public boolean hasChanged() {
		synchronized (this) {
			return this.changed;
		}
	}

	public void notifyObservers() {
		notifyObservers(null);
	}

	public void notifyObservers(Object arg) {
		Observer[] obs;

		synchronized (this) {
			if (!this.changed) {
				return;
			}
			obs = this.observers.toArray(new Observer[this.observers.size()]);
			clearChanged();
		}

		for (Observer o : obs) {
			o.update(this, arg);
		}
	}

	protected void clearChanged() {
		synchronized (this) {
			this.changed = false;
		}
	}

	protected void setChanged() {
		synchronized (this) {
			this.changed = true;
		}
	}
}
