package org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal;

import java.util.Map;

import org.ioc.commons.integration.dataaccess.dao.Dao;
import org.ioc.commons.integration.dataaccess.dao.IsDaoFactory;
import org.ioc.commons.integration.dataaccess.dao.Tx;

import com.google.gwt.core.client.Scheduler;
import com.google.web.bindery.requestfactory.shared.Request;
import com.google.web.bindery.requestfactory.shared.RequestContext;
import com.google.web.bindery.requestfactory.shared.RequestFactory;

public abstract class RFBaseDao<R extends RequestContext, D extends IsDaoFactory, F extends RequestFactory> implements
		Dao<D> {
	public abstract R getRequest(RequestProvider<F> p);

	protected final Map<String, RequestQueue> queueRequests;
	protected final RequestProvider<F> useProvider;
	protected final Scheduler scheduler;

	public RFBaseDao(RequestProvider<F> requestProvider, Map<String, RequestQueue> queueRequests) {
		this.useProvider = requestProvider;
		this.queueRequests = queueRequests;
		this.scheduler = Scheduler.get();
	}

	final public F getDataFactory() {
		return this.useProvider.getDataFactory();
	}

	final protected <T> RFDaoOperationManagerWrapper<T> getRFOperationManagerWrapper(Request<T> req) {
		boolean autoFire = this.useProvider.getAutoFire();

		final RFDaoOperationManagerWrapper<T> rfOp = new RFDaoOperationManagerWrapper<T>(req, this.queueRequests);

		if (autoFire) {
			/*
			 * Se ejecuta lo último de todo tu código PERO sin llegar a esperar
			 * a que se libere el navegador (como hace setTimeout)
			 */
			this.scheduler.scheduleFinally(new Scheduler.ScheduledCommand() {

				@Override
				public void execute() {
					rfOp.fire();
				}
			});
		}

		return rfOp;
	}

	public final RequestProvider<F> getUseProvider() {
		return useProvider;
	}

	@SuppressWarnings("unchecked")
	@Override
	public final Tx<D> usingTransaction() {
		return (this.useProvider instanceof RFTxRequestProvider) ? ((RFTxRequestProvider<D, F>) this.useProvider)
				: null;
	}

}
