package org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal;

import java.util.Map;

import org.ioc.commons.integration.dataaccess.dao.DaoOperationManager;
import org.ioc.commons.integration.dataaccess.dao.EntityDao;
import org.ioc.commons.integration.dataaccess.dao.IsDaoFactory;
import org.ioc.commons.integration.dataaccess.dao.Subscriptor;

import com.google.gwt.core.client.Scheduler;
import com.google.web.bindery.requestfactory.shared.BaseProxy;
import com.google.web.bindery.requestfactory.shared.EntityProxy;
import com.google.web.bindery.requestfactory.shared.EntityProxyId;
import com.google.web.bindery.requestfactory.shared.RequestContext;
import com.google.web.bindery.requestfactory.shared.RequestFactory;
import com.google.web.bindery.requestfactory.shared.impl.SimpleEntityProxyId;

public abstract class RFBaseEntityDao<P extends EntityProxy, R extends RequestContext, I, D extends IsDaoFactory, F extends RequestFactory>
		extends RFBaseDao<R, D, F> implements EntityDao<P, I, D> {

	private Scheduler.ScheduledCommand finallyPendingOpEnd = new Scheduler.ScheduledCommand() {

		@SuppressWarnings("unchecked")
		@Override
		public void execute() {
			if (useProvider instanceof RFTxRequestProvider) {
				((RFTxRequestProvider<D, F>) useProvider).pendingOpEnd();
			}
		}
	};
	
	protected Subscriptor<P, I> subscriptor;

	public RFBaseEntityDao(RequestProvider<F> requestProvider, Map<String, RequestQueue> queueRequests) {
		super(requestProvider, queueRequests);
	}

	@Override
	final public <O> O create(Class<O> class1) {
		return createInternal(class1);
	}

	@SuppressWarnings("unchecked")
	protected <O> O createInternal(Class<O> class1) {
		assert (this.useProvider instanceof RFTxRequestProvider) : "Sólo se puede crear una instancia bajo una transacción";
		RFTxRequestProvider<D, F> rfTx = (RFTxRequestProvider<D, F>) this.useProvider;
		R useThisContext = getRequest(rfTx);
		Class<BaseProxy> classCast = (Class<BaseProxy>) class1;
		O created = (O) useThisContext.create(classCast);
		rfTx.bindProxy((BaseProxy) created);
		return created;
	}

	@SuppressWarnings("unchecked")
	@Override
	final public DaoOperationManager<P> find(I id) {
		EntityProxyId<P> entityProxyId = (id != null) ? (EntityProxyId<P>) ((RFBaseIdImpl) id).getEntityProxyId()
				: null;

		/*
		 * Si se está intentando extraer un elemento cuyo id no pertenece a un
		 * elemento persistido en BBDD, devolvemos null felizmente.
		 */
		boolean validEntityProxyId = (entityProxyId != null && !((SimpleEntityProxyId<P>) entityProxyId).isEphemeral());

		RFDaoOperationManagerWrapper<P> om = validEntityProxyId ? getRFOperationManagerWrapper(this.getDataFactory()
				.find(entityProxyId)) : RFDaoOperationManagerWrapper.<P> getForSendingNullValues();

		return om;
	}

	@Override
	public P edit(P proxy) {
		assert (this.useProvider instanceof RFTxRequestProvider) : "Sólo se puede editar una instancia bajo una transacción";
		@SuppressWarnings("unchecked")
		RFTxRequestProvider<D, F> rfTx = ((RFTxRequestProvider<D, F>) this.useProvider);
		R useThisContext = getRequest(rfTx);
		P editing = useThisContext.edit(proxy);
		rfTx.bindProxy(editing);
		return editing;
	}

	@SuppressWarnings("unchecked")
	protected void pendingOpStart() {
		if (useProvider instanceof RFTxRequestProvider) {
			((RFTxRequestProvider<D, F>) useProvider).pendingOpStart();
		}
	}

	protected void pendingOpEnd() {
		scheduler.scheduleFinally(this.finallyPendingOpEnd);
	}

}
