package org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal;

import java.util.Map;

import org.ioc.commons.flowcontrol.operationmanager.IsOperation;
import org.ioc.commons.flowcontrol.operationmanager.OperationManagerTrigger;
import org.ioc.commons.integration.dataaccess.dao.DaoOperationManager;
import org.ioc.commons.integration.dataaccess.dao.DaoReceiver;

import com.google.web.bindery.requestfactory.shared.Request;

public class RFDaoOperationManagerWrapper<T> implements DaoOperationManager<T> {

	private Map<String, RequestQueue> queueRequests;

	private Request<T> request;

	private String queueName;

	private DaoReceiver<T> receiver;

	private String[] propertyRefs;

	private boolean sendNullValue;

	private RFDaoOperationManagerWrapper<T> boundTo;

	private String operationDescription;

	private static class OperationTrigger<O extends Enum<? extends IsOperation>> {
		private O operation;
		private OperationManagerTrigger<O> trigger;

		public OperationTrigger(O operation, OperationManagerTrigger<O> trigger) {
			this.operation = operation;
			this.trigger = trigger;
		}

		public void operationBeginning() {
			this.trigger.operationBeginning(operation);
		}

		public void operationFinished(boolean success) {
			this.trigger.operationFinished(operation, success);
		}
	}

	private OperationTrigger<? extends Enum<? extends IsOperation>> operaTrigger;

	public RFDaoOperationManagerWrapper() {
		// Nada
	}

	public RFDaoOperationManagerWrapper(Request<T> request, Map<String, RequestQueue> queueRequests) {
		this.queueRequests = queueRequests;
		this.request = request;
		this.sendNullValue = false;
	}

	public static <T> RFDaoOperationManagerWrapper<T> getForSendingNullValues() {
		RFDaoOperationManagerWrapper<T> instance = new RFDaoOperationManagerWrapper<T>();

		instance.sendNullValue = true;

		return instance;
	}

	@Override
	public DaoOperationManager<T> to(DaoReceiver<T> receiver) {
		if (this.request != null) {
			this.request.to(new RFDaoReceiver<T>(receiver) {
				@Override
				public void onEnd(boolean success) {
					super.onEnd(success);

					// if (firedOp != null) {
					// firedOp.firedOpEnd();
					// }
					if (operaTrigger != null) {
						operaTrigger.operationFinished(success);
					}
				}
			});
		}
		this.receiver = receiver;

		if (this.boundTo != null) {
			assert (this.boundTo.boundTo == null) : "No puede ver anidaciones de operaciones";

			this.boundTo.to(receiver);
		}
		return this;
	}

	@Override
	public DaoOperationManager<T> with(String... propertyRefs) {
		if (this.request != null) {
			this.request.with(propertyRefs);
		}
		this.propertyRefs = propertyRefs;

		if (this.boundTo != null) {
			assert (this.boundTo.boundTo == null) : "No puede ver anidaciones de operaciones";

			this.boundTo.with(propertyRefs);
		}

		return this;
	}

	public void fire() {
		if (this.queueRequests != null && this.queueName != null) {
			RequestQueue rq = queueRequests.get(queueName);

			if (rq == null) {
				rq = new RequestQueue();
				queueRequests.put(queueName, rq);
			}

			rq.fireOrQueue(request, new RFDaoReceiver<T>(receiver), operationDescription);
		} else if (this.request != null) {
			// if (indicators != null) {
			// this.firedOp =
			// MultiLoadingHelper.get().firingNewOp(operationDescription,
			// indicators);
			// logger.info("Disparando operación '{0}'. Indicadores? {1}",
			// operationDescription, indicators.length);
			// }

			if (this.operaTrigger != null) {
				this.operaTrigger.operationBeginning();
			}
			this.request.fire();
		} else if (this.sendNullValue && this.receiver != null) {
			this.receiver.success(null);
		}

		/*
		 * Con esto, si es disparado el enlazado, se dispara también el que lo
		 * enlazó.
		 * 
		 * Se creó a raíz del entityproxy null para el Issue 5525
		 */
		if (this.boundTo != null) {
			this.boundTo.fire();
		}
	}

	@Override
	public DaoOperationManager<T> queue(String queueName) {
		this.queueName = queueName;
		if (this.boundTo != null) {
			assert (this.boundTo.boundTo == null) : "No puede ver anidaciones de operaciones";

			this.boundTo.queue(queueName);
		}

		return this;
	}

	public void bindTo(RFDaoOperationManagerWrapper<T> sourceOm) {
		sourceOm.boundTo = null;

		if (sourceOm.propertyRefs != null) {
			this.with(sourceOm.propertyRefs);
		}
		if (sourceOm.receiver != null) {
			this.to(sourceOm.receiver);
		}
		if (sourceOm.queueName != null) {
			this.queue(sourceOm.queueName);
		}

		this.performing(sourceOm.operaTrigger);

		sourceOm.boundTo = this;
	}

	@Override
	public <O extends Enum<? extends IsOperation>> DaoOperationManager<T> performing(O operation,
			OperationManagerTrigger<O> trigger) {
		return performing(new OperationTrigger<O>(operation, trigger));
	}

	private <O extends Enum<? extends IsOperation>> DaoOperationManager<T> performing(
			OperationTrigger<O> operationTrigger) {
		this.operaTrigger = operationTrigger;

		if (this.boundTo != null) {
			assert (this.boundTo.boundTo == null) : "No puede ver anidaciones de operaciones";

			this.boundTo.performing(operationTrigger);
		}
		return this;

	}
}
