package org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.validation.ConstraintViolation;

import org.ioc.commons.ui.HasLoader;
import org.ioc.commons.utils.FormatterLogger;

import com.google.gwt.core.client.Scheduler;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.google.web.bindery.requestfactory.shared.RequestContext;
import com.google.web.bindery.requestfactory.shared.ServerFailure;

/*
 * TODO: Estudiar si es sustituible por RequestBactcher de GWT
 */
public class RequestQueue {

	private static FormatterLogger logger = FormatterLogger.getLogger(RequestQueue.class);

	private List<ContextReceiver<?>> queue = new ArrayList<ContextReceiver<?>>();
	private boolean requestInProgress = false;
	private boolean cancelNextRequestsOnFail;

	public boolean hasPendingRequests() {
		return !queue.isEmpty();
	}

	private static class ContextReceiver<T> {
		public RequestContext context;
		public Receiver<Void> contextReceiver;
		public Request<? extends T> request;
		public Receiver<T> requestReceiver;
		public Set<HasLoader> indicatorSet;
	}

	public RequestQueue() {
		// Nada
	}

	public RequestQueue(boolean cancelNextRequestsOnFail) {
		this.setCancelNextRequestsOnFail(cancelNextRequestsOnFail);
	}

	/*
	 * FIXME: Si lanzamos un RequestContext en lugar de un Request, al menos en
	 * GWT 2.4, la excepción no se captura bien porque no asocia correctamente
	 * el receiver (si depuras el código dice que viene a null). Con los Request
	 * funciona correctamente así que mientras estemos en la versión 2.4. lo
	 * dejaremos sólo con Request.
	 */
	public void fireOrQueue(RequestContext context) {
		logger.warn("¡¡¡¡FIRE OR QUEUE EN PRUEBAS!!!! SI FALLA ALGO EL ORIGEN PODRÍA ESTAR AQUÍ");
		fireOrQueue(context, null, null);
	}

	public <T> void fireOrQueue(RequestContext context, final Receiver<Void> receiver,
			final Set<HasLoader> indicatorSet) {
		if (!requestInProgress) {
			requestInProgress = true;
			logger.info("No hay peticiones en curso. Lanzamos fire()...");

			if (indicatorSet != null) {
				for (HasLoader indicator : indicatorSet) {
					indicator.setLoading(true);
				}
			}

			context.fire(new RFDaoReceiver<Void>() {

				@Override
				public void onEnd(boolean success) {
					super.onEnd(success);

					if (indicatorSet != null) {
						for (HasLoader indicator : indicatorSet) {
							indicator.setLoading(false);
						}
					}
				}

				@Override
				public void success(Void response) {
					logger.info("... fire terminado ok");

					if (receiver != null) {
						receiver.onSuccess(response);
					}

					ContextReceiver<?> nextRequest = (!queue.isEmpty() ? queue.remove(0) : null);
					requestInProgress = false;
					if (nextRequest != null) {
						fireOrQueue(nextRequest);
					}
				}

				@Override
				public void constraintViolation(Set<ConstraintViolation<?>> violations) {
					logger.info("... fire con restricciones violadas");
					requestInProgress = false;
					if (receiver != null) {
						receiver.onConstraintViolation(violations);
					} else {
						super.onConstraintViolation(violations);
					}
				}

				@Override
				public void failure(ServerFailure error) {
					logger.info("... fire con fallos");
					requestInProgress = false;
					if (receiver != null) {
						receiver.onFailure(error);
					} else {
						super.onFailure(error);
					}
				}
			});
		} else {
			logger.info("Hay peticiones en curso. Encolamos");
			ContextReceiver<T> contextReceiver = new ContextReceiver<T>();
			contextReceiver.context = context;
			contextReceiver.contextReceiver = receiver;
			contextReceiver.indicatorSet = indicatorSet;
			queue.add(contextReceiver);
		}
	}

	public <T> void fireOrQueue(Request<? extends T> request) {
		fireOrQueue(request, null);
	}

	public <T> void fireOrQueue(Request<? extends T> request, final Receiver<T> receiver) {
		fireOrQueue(request, receiver, null);
	}

	public <T> void fireOrQueue(Request<? extends T> request, final Receiver<T> receiver, String operationDescription,
			HasLoader... indicators) {
		Set<HasLoader> indicatorSet = new HashSet<HasLoader>();
		if (indicators != null) {
			for (HasLoader indicator : indicators) {
				indicatorSet.add(indicator);
			}
		}

		fireOrQueue(request, receiver, operationDescription, indicatorSet);
	}

	public <T> void fireOrQueue(Request<? extends T> request, final Receiver<T> receiver, String operationDescription,
			Set<HasLoader> indicators) {
		if (!requestInProgress) {
			requestInProgress = true;
			logger.info("No hay peticiones en curso. Lanzamos fire()...");

			//TODO: Meter aquí el operation manager
//			final FiredOp fop = (operationDescription != null && indicators != null) ? MultiLoadingHelper.get()
//					.firingNewOp(operationDescription, indicators) : null;

			request.fire(new RFDaoReceiver<T>() {

				@Override
				public void onEnd(boolean success) {
					super.onEnd(success);

					//TODO: Meter aquí el operation manager
//					if (fop != null) {
//						fop.firedOpEnd();
//					}
				}

				@Override
				public void success(T response) {
					logger.info("... fire terminado ok");

					if (receiver != null) {
						receiver.onSuccess(response);
					}

					requestInProgress = false;
					fireNext();
				}

				@Override
				public void constraintViolation(Set<ConstraintViolation<?>> violations) {
					logger.info("... fire con restricciones violadas");
					requestInProgress = false;

					if (receiver != null) {
						receiver.onConstraintViolation(violations);
					} else {
						super.constraintViolation(violations);
					}

					if (cancelNextRequestsOnFail) {
						queue.clear();
					} else {
						fireNext();
					}
				}

				@Override
				public void failure(ServerFailure error) {
					logger.info("... fire con fallos");
					requestInProgress = false;

					if (receiver != null) {
						receiver.onFailure(error);
					} else {
						super.failure(error);
					}

					if (cancelNextRequestsOnFail) {
						queue.clear();
					} else {
						fireNext();
					}
				}
			});
		} else {
			logger.info("Hay peticiones en curso. Encolamos");
			ContextReceiver<T> contextReceiver = new ContextReceiver<T>();
			contextReceiver.request = request;
			contextReceiver.requestReceiver = receiver;
			queue.add(contextReceiver);
		}
	}

	protected <T> void fireOrQueue(final ContextReceiver<T> nextRequest) {
		Scheduler.get().scheduleDeferred(new Scheduler.ScheduledCommand() {

			@Override
			public void execute() {
				if (nextRequest.context != null) {
					fireOrQueue(nextRequest.context, nextRequest.contextReceiver, nextRequest.indicatorSet);
				} else {
					fireOrQueue(nextRequest.request, nextRequest.requestReceiver);
				}
			}
		});
	}

	public boolean isCancelNextRequestsOnFail() {
		return cancelNextRequestsOnFail;
	}

	public void setCancelNextRequestsOnFail(boolean cancelNextRequestsOnFail) {
		this.cancelNextRequestsOnFail = cancelNextRequestsOnFail;
	}

	protected void fireNext() {
		ContextReceiver<?> nextRequest = (!queue.isEmpty() ? queue.remove(0) : null);
		if (nextRequest != null) {
			fireOrQueue(nextRequest);
		}
	}

}
