package org.ioc.commons.impl.gwt.integration.server.dataaccess.requestfactory.util;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

import com.google.web.bindery.requestfactory.shared.ServiceLocator;

/**
 * Implementation class for ServiceLocator which inspects by using reflection
 * inside itself all the attributes and registers them as services for using in
 * getInstance().
 * 
 * @author Jesús Lunar Pérez
 * 
 */
public abstract class ReflectionServiceLocator implements ServiceLocator {

	private Map<Class<?>, Object> registeredServices = new HashMap<Class<?>, Object>();

	public ReflectionServiceLocator() throws IllegalArgumentException, IllegalAccessException {
		this.createServiceInstances();
		this.registerAttributesAsServices();
	}

	protected void createServiceInstances() {
		// Override me
	}

	private void registerAttributesAsServices() throws IllegalArgumentException, IllegalAccessException {
		for (Field serviceField : this.getClass().getDeclaredFields()) {
			registeredServices.put(serviceField.getDeclaringClass(), serviceField.get(this));
		}

	}

	@Override
	public Object getInstance(Class<?> clazz) {
		Object serviceInstance = registeredServices.get(clazz);

		assert (serviceInstance != null) : "Service class " + clazz + " has not been declared as field on class " + this.getClass();

		return serviceInstance;
	}

}
