package org.ioc.commons.impl.gwt.rebind.dataaccess.requestfactory.generator;

import java.io.PrintWriter;
import java.util.Map;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFDaoFactory;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RequestProvider;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RequestQueue;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;

public class RFDaoFactoryGenerator extends RFGeneratorBase {

	public RFDaoFactoryGenerator() {
		// Nada
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);

		String rfDaoFactoryGeneratedSimpleClassname = typeName + RFDataManagerGenerator.GENERATED_CLASS_SUFIX;
		String rfDaoFactoryGeneratedFullClassname = rfDataManagerType.getPackage().getName() + "."
				+ rfDaoFactoryGeneratedSimpleClassname;
		SourceWriter sw = getRFDaoFactorySourceWriter(logger, context, rfDaoFactoryGeneratedSimpleClassname);

		if (sw != null) {

			sw.println();
			sw.indent();

			for (JClassType dao : daos) {
				sw.println("private %s %s;", dao.getQualifiedSourceName(),
						Format.decapitalize(dao.getSimpleSourceName()));
			}

			sw.println();
			writeConstructor(logger, context, sw, rfDaoFactoryGeneratedSimpleClassname);
			sw.println();
			for (JClassType dao : daos) {
				writeDaoGetter(logger, context, sw, dao);
			}

			sw.outdent();

			sw.commit(logger);
		}
		return rfDaoFactoryGeneratedFullClassname;
	}

	private void writeConstructor(TreeLogger logger, GeneratorContext context, SourceWriter sw,
			String rfDaoFactoryGeneratedSimpleClassname) {
		sw.println("public %s(RequestProvider<%s> requestProvider, Map<String, RequestQueue> queueRequests) {",
				rfDaoFactoryGeneratedSimpleClassname, requestFactoryClass.getName());
		sw.indent();
		sw.println("super(requestProvider, queueRequests);");
		sw.outdent();
		sw.println("}");

	}

	private void writeDaoGetter(TreeLogger logger, GeneratorContext context, SourceWriter sw, JClassType dao)
			throws UnableToCompleteException {
		RFDaoGenerator daoGenerator = new RFDaoGenerator(dao);
		String daoClassname = dao.getName();
		String daoAttribute = Format.decapitalize(daoClassname);
		String daoImplClassname = daoGenerator.generate(logger, context,
				Format.substitute("RF{0}", daoClassname));

		sw.println("@Override");
		sw.println("public %s %s() {", daoClassname, daoAttribute);
		sw.indent();
		sw.println("if (this.%s == null) {", daoAttribute);
		sw.indent();
		sw.println("this.%s = new %s(requestProvider, queueRequests);", daoAttribute, daoImplClassname);
		sw.outdent();
		sw.println("}");
		sw.println("return this.%s;", daoAttribute);
		sw.println("}");
		sw.outdent();
	}

	protected SourceWriter getRFDaoFactorySourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rfDaoFactoryGeneratedSimpleClassname) {
		JPackage classPkg = rfDataManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rfDaoFactoryGeneratedSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rfDaoFactoryGeneratedSimpleClassname);

		String[] imports = new String[] { RFDaoFactory.class.getName(), RequestProvider.class.getName(),
				Map.class.getName(), RequestQueue.class.getName(), daoFactoryClass.getQualifiedSourceName(),
				requestFactoryClass.getQualifiedSourceName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		for (JClassType dao : daos) {
			composerFactory.addImport(dao.getQualifiedSourceName());
		}

		composerFactory.setSuperclass(String.format("%s<%s>", RFDaoFactory.class.getName(),
				requestFactoryClass.getName()));
		composerFactory.addImplementedInterface(daoFactoryClass.getName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}
}
