package org.ioc.commons.impl.gwt.rebind.dataaccess.requestfactory.generator;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.RFMapping;
import org.ioc.commons.integration.dataaccess.dao.Dao;
import org.ioc.commons.integration.dataaccess.dao.EntityDao;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.ext.Generator;
import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JMethod;
import com.google.gwt.core.ext.typeinfo.JParameterizedType;
import com.google.gwt.core.ext.typeinfo.JType;
import com.google.gwt.core.ext.typeinfo.TypeOracle;

abstract class RFGeneratorBase extends Generator {
	
	public static class DaoMapping {

		public JClassType id;
		public JClassType proxy;
		public JClassType request;
		
	}

	protected static JClassType rfDataManagerType;
	protected static JClassType daoFactoryClass;
	protected static JClassType requestFactoryClass;
	protected static Set<JClassType> daos;
	protected static Set<JClassType> entityDaos;
	protected static Set<JClassType> requests;
	protected static final Map<JClassType,DaoMapping> daoMapping = new HashMap<JClassType, RFGeneratorBase.DaoMapping>();

	public RFGeneratorBase() {
		// Nada
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		TypeOracle typeOracle = context.getTypeOracle();
		assert (typeOracle != null);

		if (rfDataManagerType == null) {
			rfDataManagerType = typeOracle.findType(typeName);
			if (rfDataManagerType == null) {
				logger.log(TreeLogger.ERROR, "No se pudieron encontrar metadatos para el tipo '" + typeName + "'", null);
				throw new UnableToCompleteException();
			}

			JParameterizedType parametrized = rfDataManagerType.getSuperclass().isParameterized();
			JClassType[] types = parametrized.getTypeArgs();
			daoFactoryClass = types[0];
			requestFactoryClass = types[1];

			JClassType entityDaoType = typeOracle.findType(EntityDao.class.getName());
			JClassType daoType = typeOracle.findType(Dao.class.getName());

			daos = new HashSet<JClassType>();
			entityDaos = new HashSet<JClassType>();
			requests = new HashSet<JClassType>();

			for (JMethod method : daoFactoryClass.getMethods()) {
				JType returnType = method.getReturnType();
				JClassType returnClass = returnType.isInterface();
				if (returnClass != null) {
					if (returnClass.isAssignableTo(entityDaoType)) {
						entityDaos.add(returnClass);
					}

					if (returnClass.isAssignableTo(daoType)) {
						daos.add(returnClass);
					}
				}
			}
			
			RFMapping mapping = rfDataManagerType.getAnnotation(RFMapping.class);

			if (mapping == null) {
				logger.log(TreeLogger.ERROR, Format.substitute(
						"The class '{0}' must be annotated by the annotation {1}", typeName, RFMapping.class), null);
				throw new UnableToCompleteException();
			}

			for (RFMapping.EntityDao dao : mapping.entityDaos()) {
				JClassType type = typeOracle.findType(dao.entityDao().getCanonicalName());
				if (!daos.contains(type)) {
					logger.log(
							TreeLogger.WARN,
							Format.substitute(
									"The class '{0}' contains an annotation for the {1} entity dao but there isnt a method declared on the '{2}' class",
									rfDataManagerType, type, daoFactoryClass), null);
				}

				DaoMapping daoMapping = new DaoMapping();
				JClassType idCT = typeOracle.findType(dao.id().getCanonicalName());
				JClassType proxyCT = typeOracle.findType(dao.proxy().getCanonicalName());
				JClassType requestCT = typeOracle.findType(dao.request().getCanonicalName());

				daoMapping.id = idCT;
				daoMapping.proxy = proxyCT;
				daoMapping.request = requestCT;

				RFGeneratorBase.daoMapping.put(type, daoMapping);
			}

			for (RFMapping.Dao dao : mapping.plainDaos()) {
				JClassType type = typeOracle.findType(dao.dao().getCanonicalName());
				if (!daos.contains(type)) {
					logger.log(
							TreeLogger.WARN,
							Format.substitute(
									"The class '{0}' contains an annotation for the {1} dao but there isnt a method declared on the '{2}' class",
									rfDataManagerType, type, daoFactoryClass), null);
				}

				DaoMapping daoMapping = new DaoMapping();
				JClassType requestCT = typeOracle.findType(dao.request().getCanonicalName());

				daoMapping.request = requestCT;

				RFGeneratorBase.daoMapping.put(type, daoMapping);
			}

			for(JMethod method : requestFactoryClass.getMethods())  {
				requests.add(method.getReturnType().isClassOrInterface());
			}
		}

		return null;
	}

}
