package org.ioc.commons.impl.gwt.rebind.dataaccess.requestfactory.generator;

import java.io.PrintWriter;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFBaseIdImpl;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;
import com.google.web.bindery.requestfactory.shared.EntityProxyId;

public class RFIdGenerator extends RFGeneratorBase {

	private DaoMapping daoMap;

	public RFIdGenerator(DaoMapping daoMapping) {
		this.daoMap = daoMapping;
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);

		String rfIdGeneratedSimpleClassname = typeName + RFDataManagerGenerator.GENERATED_CLASS_SUFIX;
		String rfIdGeneratedFullClassname = rfDataManagerType.getPackage().getName() + "."
				+ rfIdGeneratedSimpleClassname;

		SourceWriter sw = getRFIdSourceWriter(logger, context, rfIdGeneratedSimpleClassname);

		if (sw != null) {

			sw.println("public %s(EntityProxyId<%s> entityProxyId) {", rfIdGeneratedSimpleClassname,
					daoMap.proxy.getName());
			sw.println("	super(entityProxyId);");
			sw.println("}");

			sw.commit(logger);
		}

		return rfIdGeneratedFullClassname;
	}

	private SourceWriter getRFIdSourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rfIdGeneratedSimpleClassname) throws UnableToCompleteException {
		JPackage classPkg = rfDataManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rfIdGeneratedSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rfIdGeneratedSimpleClassname);

		String[] imports = new String[] { EntityProxyId.class.getName(), RFBaseIdImpl.class.getName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		composerFactory.setSuperclass(RFBaseIdImpl.class.getName());

		composerFactory.addImport(daoMap.proxy.getQualifiedSourceName());
		composerFactory.addImport(daoMap.id.getQualifiedSourceName());

		composerFactory.addImplementedInterface(daoMap.id.getName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}

}
