package org.ioc.commons.impl.gwt.rebind.dataaccess.requestfactory.generator;

import java.io.PrintWriter;
import java.util.Map;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFTxRequestProvider;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RequestQueue;
import org.ioc.commons.integration.dataaccess.dao.Tx;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;
import com.google.web.bindery.requestfactory.shared.BaseProxy;
import com.google.web.bindery.requestfactory.shared.RequestContext;

public class RFTxRequestProviderGenerator extends RFGeneratorBase {

	private String daoFactoryGeneratedName;

	public RFTxRequestProviderGenerator(String daoFactoryGeneratedName) {
		this.daoFactoryGeneratedName = daoFactoryGeneratedName;
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);
		String rfTxRequestSimpleClassname = typeName + RFDataManagerGenerator.GENERATED_CLASS_SUFIX;
		String rfTxRequestFullClassname = rfDataManagerType.getPackage().getName() + "." + rfTxRequestSimpleClassname;

		SourceWriter sw = getRFTxRequestProviderSourceWriter(logger, context, rfTxRequestSimpleClassname);

		if (sw != null) {
			for (JClassType requestClass : requests) {
				sw.println("private %s %s;", requestClass.getName(),
						Format.decapitalize(requestClass.getSimpleSourceName()));
			}

			sw.println("private %s daoFactory;", daoFactoryClass.getName());

			sw.println(
					"public %s(%s dataFactory, String transactionName,Map<String, RequestQueue> queueRequests, Map<BaseProxy, Tx<%s>> transactionMap) {",
					rfTxRequestSimpleClassname, requestFactoryClass.getName(), daoFactoryClass.getName());
			sw.println("	super(dataFactory, transactionName, queueRequests, transactionMap);");
			sw.println("}");
			sw.println();

			sw.println("@Override");
			sw.println("public %s daoFactory() {", daoFactoryClass.getName());
			sw.println("	if (this.daoFactory == null) {");
			sw.println("		this.daoFactory = new %s(this, queueRequests);", daoFactoryGeneratedName);
			sw.println("	}");
			sw.println("	return this.daoFactory;");
			sw.println("}");

			sw.println("@Override");
			sw.println("public <R extends RequestContext> R getRequest(Class<R> clazz) {");
			sw.indent();
			boolean first = true;
			for (JClassType requestClass : requests) {
				if (!first) {
					sw.print(" else ");
				}
				sw.println("if (clazz.equals(%s.class)) {", requestClass.getSimpleSourceName());
				sw.println("		return (R) this.get%s();", requestClass.getSimpleSourceName());
				sw.println("}");
				first = false;
			}
			sw.println("		throw new IllegalArgumentException(\"The RequestContext \" + clazz + \" has not been registered.\");");
			sw.outdent();
			sw.println("}");
			sw.println();
			for (JClassType requestClass : requests) {
				String decapitalized = Format.decapitalize(requestClass.getSimpleSourceName());
				sw.println("public %1$s get%1$s() {", requestClass.getSimpleSourceName());
				sw.println("if (this.%s == null) {", decapitalized);
				sw.println("	this.%1$s = appendNewRequest(this.dataFactory.%1$s());", decapitalized);
				sw.println("}");

				sw.println("	return this.%s;", decapitalized);
				sw.println("}");
			}
			sw.commit(logger);
		}

		return rfTxRequestFullClassname;
	}

	private SourceWriter getRFTxRequestProviderSourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rfNoTxRequestSimpleClassname) throws UnableToCompleteException {
		JPackage classPkg = rfDataManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rfNoTxRequestSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rfNoTxRequestSimpleClassname);

		String[] imports = new String[] { RequestContext.class.getName(), RFTxRequestProvider.class.getName(),
				daoFactoryClass.getQualifiedSourceName(), requestFactoryClass.getQualifiedSourceName(),
				BaseProxy.class.getName(), Tx.class.getName(), Map.class.getName(), RequestQueue.class.getName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		composerFactory.setSuperclass(String.format("%s<%s,%s>", RFTxRequestProvider.class.getName(),
				daoFactoryClass.getQualifiedSourceName(), requestFactoryClass.getQualifiedSourceName()));

		for (JClassType requestClass : requests) {
			composerFactory.addImport(requestClass.getQualifiedSourceName());
		}

		return composerFactory.createSourceWriter(ctx, printWriter);
	}
}
