package org.ioc.commons.impl.gwt.rebind.service.rpc.generator;

import java.util.HashMap;
import java.util.Map;

import org.ioc.commons.impl.gwt.integration.client.service.rpc.RpcMapping;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.ext.Generator;
import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JParameterizedType;
import com.google.gwt.core.ext.typeinfo.TypeOracle;

abstract class RpcGeneratorBase extends Generator {

	public static class ServiceMapping {

		public JClassType service;
		public JClassType rpc;

	}

	protected static JClassType rpcServiceManagerType;
	protected static JClassType serviceFactoryClass;
	protected static final Map<JClassType, ServiceMapping> serviceMapping = new HashMap<JClassType, RpcGeneratorBase.ServiceMapping>();

	public RpcGeneratorBase() {
		// Nada
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		TypeOracle typeOracle = context.getTypeOracle();
		assert (typeOracle != null);

		if (rpcServiceManagerType == null) {
			rpcServiceManagerType = typeOracle.findType(typeName);
			if (rpcServiceManagerType == null) {
				logger.log(TreeLogger.ERROR, "Unable to find metadata for type '" + typeName + "'", null);
				throw new UnableToCompleteException();
			}

			JParameterizedType parametrized = rpcServiceManagerType.getSuperclass().isParameterized();
			JClassType[] types = parametrized.getTypeArgs();
			serviceFactoryClass = types[0];

			RpcMapping mapping = rpcServiceManagerType.getAnnotation(RpcMapping.class);

			if (mapping == null) {
				logger.log(TreeLogger.ERROR, Format.substitute(
						"The class '{0}' must be annotated by the annotation {1}", typeName, RpcMapping.class), null);
				throw new UnableToCompleteException();
			}

			for (RpcMapping.Service service : mapping.value()) {
				JClassType type = typeOracle.findType(service.service().getCanonicalName());

				ServiceMapping serviceMapping = new ServiceMapping();
				JClassType serviceCT = typeOracle.findType(service.service().getCanonicalName());
				JClassType rpcCT = typeOracle.findType(service.rpc().getCanonicalName());

				serviceMapping.service = serviceCT;
				serviceMapping.rpc = rpcCT;

				RpcGeneratorBase.serviceMapping.put(type, serviceMapping);
			}

		}

		return null;
	}

}
