package org.ioc.commons.impl.gwt.rebind.service.rpc.generator;

import java.io.PrintWriter;
import java.util.Map;

import org.ioc.commons.impl.gwt.integration.client.service.rpc.rebind.internal.RpcServiceFactory;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.TreeLogger.Type;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;

public class RpcServiceFactoryGenerator extends RpcGeneratorBase {

	public RpcServiceFactoryGenerator() {
		// Nada
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);

		String rpcServiceFactoryGeneratedSimpleClassname = typeName + RpcServiceManagerGenerator.GENERATED_CLASS_SUFIX;
		String rpcServiceGeneratedFullClassname = rpcServiceManagerType.getPackage().getName() + "."
				+ rpcServiceFactoryGeneratedSimpleClassname;
		SourceWriter sw = getRFDaoFactorySourceWriter(logger, context, rpcServiceFactoryGeneratedSimpleClassname);

		if (sw != null) {

			sw.println();
			sw.indent();

			for (JClassType service : serviceMapping.keySet()) {
				sw.println("private %s %s;", service.getQualifiedSourceName(),
						Format.decapitalize(service.getSimpleSourceName()));
			}

			sw.println();
			writeConstructor(logger, context, sw, rpcServiceFactoryGeneratedSimpleClassname);
			sw.println();
			for (JClassType service : serviceMapping.keySet()) {
				writeServiceGetter(logger, context, sw, serviceMapping.get(service));
			}

			sw.outdent();

			sw.commit(logger);
		}
		return rpcServiceGeneratedFullClassname;
	}

	private void writeConstructor(TreeLogger logger, GeneratorContext context, SourceWriter sw,
			String rfDaoFactoryGeneratedSimpleClassname) {
		sw.println("public %s() {", rfDaoFactoryGeneratedSimpleClassname);
		sw.indent();
		sw.println("super();");
		sw.outdent();
		sw.println("}");

	}

	private void writeServiceGetter(TreeLogger logger, GeneratorContext context, SourceWriter sw, ServiceMapping serviceMapping)
			throws UnableToCompleteException {
		
		RpcServiceGenerator serviceGenerator = new RpcServiceGenerator(serviceMapping.service);
		String serviceClassname = serviceMapping.service.getName();
		String serviceAttribute = Format.decapitalize(serviceClassname);
		String serviceImplClassname = serviceGenerator.generate(logger, context,
				Format.substitute("Rpc{0}", serviceClassname));

		logger.log(Type.INFO, "Writting " + serviceImplClassname);
		
		sw.println("@Override");
		sw.println("public %s %s() {", serviceClassname, serviceAttribute);
		sw.indent();
		sw.println("if (this.%s == null) {", serviceAttribute);
		sw.indent();
		sw.println("this.%1$s = new %2$s(GWT.<%3$s>create(%4$s.class));", serviceAttribute, serviceImplClassname,
				serviceMapping.rpc.getQualifiedSourceName()+ "Async",serviceMapping.rpc.getQualifiedSourceName());
		sw.outdent();
		sw.println("}");
		sw.println("return this.%s;", serviceAttribute);
		sw.println("}");
		sw.outdent();
	}

	protected SourceWriter getRFDaoFactorySourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rpcServiceFactoryGeneratedSimpleClassname) {
		JPackage classPkg = rpcServiceManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rpcServiceFactoryGeneratedSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rpcServiceFactoryGeneratedSimpleClassname);

		String[] imports = new String[] { RpcServiceFactory.class.getName(), Map.class.getName(),
				serviceFactoryClass.getQualifiedSourceName(), GWT.class.getName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		for (ServiceMapping serviceMap : serviceMapping.values()) {
			composerFactory.addImport(serviceMap.service.getQualifiedSourceName());
			composerFactory.addImport(serviceMap.rpc.getQualifiedSourceName());
		}

		composerFactory.setSuperclass(String.format("%s", RpcServiceFactory.class.getName()));
		composerFactory.addImplementedInterface(serviceFactoryClass.getName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}
}
