package org.ioc.commons.impl.gwt.rebind.service.rpc.generator;

import java.io.PrintWriter;
import java.util.Map;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFBaseIdImpl;
import org.ioc.commons.impl.gwt.integration.client.service.rpc.rebind.internal.RpcBaseService;
import org.ioc.commons.impl.gwt.integration.client.service.rpc.rebind.internal.RpcServiceFactory;
import org.ioc.commons.impl.gwt.integration.client.service.rpc.rebind.internal.RpcServiceOperationManagerWrapper;
import org.ioc.commons.integration.service.ServiceCallback;
import org.ioc.commons.integration.service.ServiceOperationManager;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.TreeLogger.Type;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JMethod;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.core.ext.typeinfo.JParameter;
import com.google.gwt.core.ext.typeinfo.JParameterizedType;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;

public class RpcServiceGenerator extends RpcGeneratorBase {

	private JClassType service;

	public RpcServiceGenerator(JClassType service) {
		this.service = service;
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);

		String rpcServiceGeneratedSimpleClassname = typeName + RpcServiceManagerGenerator.GENERATED_CLASS_SUFIX;
		String rpcServiceGeneratedFullClassname = rpcServiceManagerType.getPackage().getName() + "."
				+ rpcServiceGeneratedSimpleClassname;

		logger.log(Type.INFO, Format.substitute("Generating service {0}...", rpcServiceGeneratedSimpleClassname));

		writeService(logger, context, rpcServiceGeneratedSimpleClassname);

		return rpcServiceGeneratedFullClassname;
	}

	protected void writeService(TreeLogger logger, GeneratorContext context, String rpcServiceGeneratedSimpleClassname)
			throws UnableToCompleteException {
		SourceWriter sw = getRpcServiceSourceWriter(logger, context, rpcServiceGeneratedSimpleClassname);

		if (sw != null) {
			ServiceMapping serviceMapping = this.getServiceMapping(logger);

			sw.println("public %s(%s rpc) {", rpcServiceGeneratedSimpleClassname,
					serviceMapping.rpc.getQualifiedSourceName() + "Async");
			sw.println("	super(rpc);");
			sw.println("}");
			sw.println();

			for (JMethod method : this.service.getMethods()) {
				writeServiceMethod(method, logger, context, rpcServiceGeneratedSimpleClassname, sw);
			}

			sw.commit(logger);
		}
	}

	private SourceWriter getRpcServiceSourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rpcServiceGeneratedSimpleClassname) throws UnableToCompleteException {
		JPackage classPkg = rpcServiceManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rpcServiceGeneratedSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rpcServiceGeneratedSimpleClassname);

		String[] imports = new String[] { RpcServiceFactory.class.getName(), Map.class.getName(),
				serviceFactoryClass.getQualifiedSourceName(),

				RpcServiceOperationManagerWrapper.class.getName(), RFBaseIdImpl.class.getName(),
				ServiceCallback.class.getName(), Scheduler.class.getName(), ServiceOperationManager.class.getName(), AsyncCallback.class.getName()};
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		ServiceMapping serviceMapping = getServiceMapping(logger);

		composerFactory.setSuperclass(String.format("%s<%s>", RpcBaseService.class.getName(),
				serviceMapping.rpc.getQualifiedSourceName()+"Async"));

		composerFactory.addImport(serviceMapping.rpc.getQualifiedSourceName());

		composerFactory.addImplementedInterface(service.getQualifiedSourceName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}

	protected ServiceMapping getServiceMapping(TreeLogger logger) throws UnableToCompleteException {
		ServiceMapping serviceMapping = RpcGeneratorBase.serviceMapping.get(this.service);

		if (serviceMapping == null) {
			logger.log(TreeLogger.ERROR, Format.substitute(
					"There is no mapping for the service '{0}' on the class {1}", service, rpcServiceManagerType), null);
			throw new UnableToCompleteException();
		}
		return serviceMapping;
	}

	private void writeServiceMethod(JMethod method, TreeLogger logger, GeneratorContext context,
			String rpcServiceGeneratedSimpleClassname, SourceWriter sw) throws UnableToCompleteException {

		JParameterizedType returnParametrized = method.getReturnType().isParameterized();

		sw.print("public ServiceOperationManager<%s> %s(", returnParametrized.getTypeArgs()[0].getParameterizedQualifiedSourceName(),
				method.getName());
		writeParameters(method.getParameters(), logger, context, rpcServiceGeneratedSimpleClassname, sw);
		sw.println(") {");
		sw.println();
		sw.println(
				"	final RpcServiceOperationManagerWrapper<%1$s> om = new RpcServiceOperationManagerWrapper<%1$s>();",
				returnParametrized.getTypeArgs()[0].getParameterizedQualifiedSourceName());
		sw.println();
		sw.println("	Scheduler.get().scheduleFinally(new Scheduler.ScheduledCommand() {");
		sw.println();
		sw.println("		@Override");
		sw.println("		public void execute() {");
		sw.println("			if (om.getOperaTrigger() != null) {");
		sw.println("				om.getOperaTrigger().operationBeginning();");
		sw.println("			}");
		sw.println();
		sw.println("			rpc.%s(new AsyncCallback<%s>() {", method.getName(), returnParametrized.getTypeArgs()[0].getParameterizedQualifiedSourceName());
		sw.println();
		sw.println("				@Override");
		sw.println("				public void onSuccess(%s result) {", returnParametrized.getTypeArgs()[0].getParameterizedQualifiedSourceName());
		sw.println("					handleSuccess(om.getCallback(), result, om.getOperaTrigger());");
		sw.println("				}");
		sw.println();
		sw.println("				@Override");
		sw.println("				public void onFailure(Throwable caught) {");
		sw.println("					handleFail(om.getCallback(), caught, om.getOperaTrigger());");
		sw.println("				}");
		sw.println();
		sw.println("			});");
		sw.println("		}");
		sw.println("	});");
		sw.println();
		sw.println("	return om;");
		sw.println("}");

	}

	private void writeParameters(JParameter[] parameters, TreeLogger logger, GeneratorContext context,
			String rpcServiceGeneratedSimpleClassname, SourceWriter sw) {

		for (int i = 0; i < parameters.length; i++) {
			if (i != 0) {
				sw.print(", ");
			}
			JParameter parameter = parameters[i];

			sw.print(parameter.getType().getParameterizedQualifiedSourceName());
			sw.print(" ");
			sw.print(parameter.getName());
		}

	}

}
