package org.ioc.commons.impl.gwt.rebind.service.rpc.generator;

import java.io.PrintWriter;

import org.ioc.commons.impl.gwt.integration.client.service.rpc.RpcServiceManager;
import org.ioc.commons.impl.gwt.integration.client.service.rpc.rebind.internal.RpcServiceFactory;
import org.ioc.commons.utils.Format;
import org.ioc.commons.utils.FormatterLogger;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.TreeLogger.Type;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;

public class RpcServiceManagerGenerator extends RpcGeneratorBase {

	static final String GENERATED_CLASS_SUFIX = "Generated";
	private String serviceFactoryGeneratedName;

	public RpcServiceManagerGenerator() {
		// Nada
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);

		String rfServiceManagerSimpleSourceName = rpcServiceManagerType.getSimpleSourceName();
		String rfServiceManagerGeneratedFullClassname = typeName + GENERATED_CLASS_SUFIX;

		logger.log(Type.INFO, Format.substitute("Generating RPC service wrappers for {0} service manager.",
				rfServiceManagerSimpleSourceName));

		SourceWriter sw = getRFDataManagerSourceWriter(logger, context);

		if (sw != null) {

			sw.println();
			sw.indent();
			sw.println("private static final FormatterLogger logger = FormatterLogger.getLogger(%s.class);",
					rfServiceManagerSimpleSourceName + GENERATED_CLASS_SUFIX);
			sw.println();
			writeServiceFactory(logger, context, sw);
			sw.println();
			sw.outdent();

			sw.commit(logger);
		}

		return rfServiceManagerGeneratedFullClassname;
	}

	private void writeServiceFactory(TreeLogger logger, GeneratorContext context, SourceWriter sw)
			throws UnableToCompleteException {

		sw.println("@Override");
		sw.println("public %s serviceFactory() {", serviceFactoryClass.getSimpleSourceName());
		sw.indent();

		this.serviceFactoryGeneratedName = new RpcServiceFactoryGenerator().generate(logger, context,
				RpcServiceFactory.class.getSimpleName());

		sw.println("return new %s();", this.serviceFactoryGeneratedName);
		sw.outdent();
		sw.println("}");
	}

	protected SourceWriter getRFDataManagerSourceWriter(TreeLogger logger, GeneratorContext ctx) {
		JPackage classPkg = rpcServiceManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		String generatedClassSimpleName = rpcServiceManagerType.getName() + GENERATED_CLASS_SUFIX;
		PrintWriter printWriter = ctx.tryCreate(logger, packageName, generatedClassSimpleName);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				generatedClassSimpleName);

		String[] imports = new String[] { RpcServiceManager.class.getName(), FormatterLogger.class.getName(),
				serviceFactoryClass.getQualifiedSourceName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		composerFactory.setSuperclass(rpcServiceManagerType.getQualifiedSourceName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}
}
