package org.ioc.commons.impl.gwt.shared.common.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Observable;
import java.util.Observer;

public class ObservableArrayList<E> extends Observable implements List<E> {

	private List<E> dataList = new ArrayList<E>();

	public ObservableArrayList(Observer... observers) {
		addObservers(observers);
	}

	public void addObservers(Observer... observers) {
		if (observers != null) {
			for (Observer observer : observers) {
				this.addObserver(observer);
			}
		}
	}

	public void deleteObservers(Observer... observers) {
		if (observers != null) {
			for (Observer observer : observers) {
				this.deleteObserver(observer);
			}
		}
	}

	@Override
	public int size() {
		return dataList.size();
	}

	@Override
	public boolean isEmpty() {
		return dataList.isEmpty();
	}

	@Override
	public boolean contains(Object o) {
		return dataList.contains(o);
	}

	@Override
	public Iterator<E> iterator() {
		return dataList.iterator();
	}

	@Override
	public Object[] toArray() {
		return dataList.toArray();
	}

	@Override
	public <T> T[] toArray(T[] a) {
		return dataList.toArray(a);
	}

	@Override
	public boolean add(E e) {
		boolean add = dataList.add(e);
		this.setChanged();
		notifyObservers();
		return add;
	}

	@Override
	public boolean remove(Object o) {
		boolean remove = dataList.remove(o);
		this.setChanged();
		notifyObservers();
		return remove;
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		return dataList.containsAll(c);
	}

	@Override
	public boolean addAll(Collection<? extends E> c) {
		boolean addAll = dataList.addAll(c);
		this.setChanged();
		notifyObservers();
		return addAll;
	}

	@Override
	public boolean addAll(int index, Collection<? extends E> c) {
		boolean addAll = dataList.addAll(index, c);
		this.setChanged();
		notifyObservers();
		return addAll;
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		boolean removeAll = dataList.removeAll(c);
		this.setChanged();
		notifyObservers();
		return removeAll;
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		boolean retainAll = dataList.retainAll(c);
		this.setChanged();
		notifyObservers();
		return retainAll;
	}

	@Override
	public void clear() {
		dataList.clear();
		this.setChanged();
		notifyObservers();
	}

	@Override
	public E get(int index) {
		return dataList.get(index);
	}

	@Override
	public E set(int index, E element) {
		E set = dataList.set(index, element);
		this.setChanged();
		notifyObservers();
		return set;
	}

	@Override
	public void add(int index, E element) {
		dataList.add(index, element);
		this.setChanged();
		notifyObservers();
	}

	@Override
	public E remove(int index) {
		E remove = dataList.remove(index);
		this.setChanged();
		notifyObservers();
		return remove;
	}

	@Override
	public int indexOf(Object o) {
		return dataList.indexOf(o);
	}

	@Override
	public int lastIndexOf(Object o) {
		return dataList.lastIndexOf(o);
	}

	@Override
	public ListIterator<E> listIterator() {
		return dataList.listIterator();
	}

	@Override
	public ListIterator<E> listIterator(int index) {
		return dataList.listIterator(index);
	}

	@Override
	public List<E> subList(int fromIndex, int toIndex) {
		return dataList.subList(fromIndex, toIndex);
	}

}
