package org.ioc.commons.integration.dataaccess.dao;

import java.util.Set;

import javax.validation.ConstraintViolation;

/**
 * A dao data receiver.
 * 
 * @author Jesús Lunar Pérez
 * 
 * @param <V>
 *            Data type
 */
public interface DaoReceiver<V> {

	/**
	 * The first method called when data has been received.
	 * 
	 * @param success
	 *            true if the data was received OK; false otherwise.
	 */
	void onEnd(boolean success);

	/**
	 * Method called when data was received sucessfuly
	 * 
	 * @param response
	 *            Data received
	 */
	void success(V response);

	/**
	 * Method called when data was not receive due to a server failure (handled
	 * error).
	 * 
	 * @param serverFailure
	 */
	void failure(ServerFailure serverFailure);

	/**
	 * Method called when data was not receive due to an exception (not handled
	 * error).
	 * 
	 * @param caught
	 *            Exception thrown
	 */
	void failure(Throwable caught);

	/**
	 * Method called when some constraints have been violated trying to make the
	 * DAO operation.
	 * 
	 * @param violations
	 */
	void constraintViolation(Set<ConstraintViolation<?>> violations);

}
