package org.ioc.commons.integration.dataaccess.dao;

public interface DataManager<D extends IsDaoFactory> {
	/**
	 * It opens a transaction under a given name. If a transaction with the same
	 * name already exists, it will be closed.
	 * 
	 * Implementations of this method must ensure that it will only exist one
	 * opened transaction under this name at the same time, by closing through
	 * closeTransaction() method the previous one.
	 * 
	 * @param transactionName
	 *            A transaction name
	 * @return The opened transaction
	 */
	public Tx<D> openTransaction(String transactionName);

	/**
	 * @return An instance of the Dao factory.
	 */
	public D daoFactory();

	/**
	 * Determines if the specified named queue has or not pending requests.
	 * 
	 * @param queueName
	 * @return true if it has pending request; false otherwise.
	 */
	public boolean hasQueuePendingRequests(String queueName);

	/**
	 * It returns the transaction used to create/edit the object passed as
	 * parameter, if exists.
	 * 
	 * @param dataObject
	 *            Data object
	 * @return Transaction used for creating/editing the data object or null.
	 */
	public <P> Tx<D> getTransactionOwner(P dataObject);

	/**
	 * Useful tools for objects returned by DAOs.
	 * 
	 * @return Dao tools implementation
	 */
	public DaoTools daoTools();
}
