package org.ioc.commons.integration.dataaccess.dao;

/**
 * A dao for an entity object.
 * 
 * @author Jesús Lunar Pérez
 * 
 * @param <T>
 *            Entity object type
 * @param <I>
 *            Identifier type
 * @param <D>
 *            Dao factory type
 */
public interface EntityDao<T, I, D extends IsDaoFactory> extends Dao<D> {
	/**
	 * The identifier of the entity objects
	 * 
	 * @param entity
	 *            Entity object
	 * @return The identifier for the entity object.
	 */
	I getId(Object entity);

	/**
	 * Find an entity object through its identifier.
	 * 
	 * @param id
	 *            Unique identifier of the object.
	 * @return An operation manager for handling this operation.
	 */
	DaoOperationManager<T> find(I id);

	/**
	 * Creates a new entity object.
	 * 
	 * Only DAOs under a transaction will return values here; otherwise it will
	 * throw an exception.
	 * 
	 * @return new entity object.
	 */
	T create();

	/**
	 * Edit an entity object
	 * 
	 * @param entity
	 *            Entity for starting edition
	 * @return Entity object ready for editing.
	 */
	T edit(T entity);

	/**
	 * Creates a new secondary object used from the entity. Useful for creating
	 * object which are not entities but are needed for the entity, such as
	 * relationship objects that will be set to the entity.
	 * 
	 * @param class1
	 *            Class of the new secondary object
	 * @return An instance for a new secondary object.
	 */
	<O> O create(Class<O> class1);

	/**
	 * It returns a subscriptor for operations made by the DAO
	 * 
	 * @return Subscriptor
	 */
	Subscriptor<T, I> subscriptor();
}
