package org.ioc.commons.integration.dataaccess.dao;

/**
 * A transaction opened from a Data manager
 * 
 * @author Jesús Lunar Pérez
 * 
 * @param <D>
 *            Dao factory type
 */
public interface Tx<D extends IsDaoFactory> {
	public D daoFactory();

	/**
	 * It fires all operations accumulated into the transaction.
	 * 
	 * The transaction becomes invalid once the method
	 * {@link DaoReceiver#onEnd(boolean)} will be fired in whichever of the
	 * accumulated operations, but never before that.
	 * 
	 * In case a constraint violation happens, the transaction will remain valid
	 * and must be reused for firing the accumulated operations.
	 */
	public void fireTransaction();

	/**
	 * It fires operations as {@link #fireTransaction()} but uses internally a queue for firing it.
	 * 
	 * Read more about queue on {@link DaoOperationManager#queue(String)}
	 * 
	 * @param queueName
	 */
	public void fireOrQueueTransaction(String queueName);

	/**
	 * Indica si una transacción puede ser lanzada.
	 */
	boolean canBeFired();
}
