package org.ioc.commons.utils;

import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

public class FormatterLogger {

	protected Logger logger;

	protected FormatterLogger(Logger logger) {
		this.logger = logger;
	}

	public static String substitute(String text, Object... params) {
		for (int i = 0; i < params.length; i++) {
			Object p = params[i];
			text = text.replaceAll("\\{" + i + "}", safeRegexReplacement(String.valueOf(p)));
		}
		return text;
	}

	protected static String safeRegexReplacement(String replacement) {
		if (replacement == null) {
			return "";
		}

		return replacement.replaceAll("\\\\", "\\\\\\\\").replaceAll("\\$", "\\\\\\$");
	}

	public static FormatterLogger getLogger(Class<?> clazz) {
		return new FormatterLogger(Logger.getLogger(clazz.getName()));
	}

	public void debug(String string, Object... params) {
		if (logger.isLoggable(Level.FINE)) {
			logger.fine(substitute(string, params));
		}
	}

	public void info(String string, Object... params) {
		if (logger.isLoggable(Level.INFO)) {
			logger.info(substitute(string, params));
		}
	}

	public void info(boolean cond, String string, Object... params) {
		if (cond) {
			info(string, params);
		}
	}

	public void error(String string, Object... params) {
		if (logger.isLoggable(Level.SEVERE)) {
			logger.log(Level.SEVERE, substitute(string, params));
		}
	}

	public void error(Throwable e, String string, Object... params) {
		if (logger.isLoggable(Level.SEVERE)) {
			logger.log(Level.SEVERE, substitute(string, params), e);
		}
	}

	public void warn(String string, Object... params) {
		if (logger.isLoggable(Level.WARNING)) {
			logger.warning(substitute(string, params));
		}
	}

	public void warn(Throwable e, String string, Object... params) {
		if (logger.isLoggable(Level.WARNING)) {
			logger.log(Level.WARNING, substitute(string, params), e);
		}
	}

	public void error(Throwable e) {
		if (logger.isLoggable(Level.SEVERE)) {
			logger.log(Level.SEVERE, null, e);
		}
	}

	public void entering(String sourceMethod, String msg, Object... params) {
		if (logger.isLoggable(Level.FINE)) {
			logger.fine(substitute("{0} - Inicio. {1}", sourceMethod, substitute(msg, params)));
		}
	}

	public void entering(String sourceMethod) {
		if (logger.isLoggable(Level.FINE)) {
			logger.fine(substitute("{0} - Inicio", sourceMethod));
		}
	}

	public void exiting(String sourceMethod, String msg, Object... result) {
		if (logger.isLoggable(Level.FINE)) {
			logger.fine(substitute("{0} - Fin. {1}", sourceMethod, substitute(msg, result)));
		}
	}

	public void exiting(String sourceMethod) {
		if (logger.isLoggable(Level.FINE)) {
			logger.fine(substitute("{0} - Fin", sourceMethod));
		}
	}

	public void addHandler(Handler handler) {
		logger.addHandler(handler);
	}

}
